@@ -76,19 +76,19 @@ use Font::TTF::Utils;
 
 sub init
 {
-    my ($k, $v, $c, $n, $i);
+    my ($k, $v, $c, $n, $i, $t);
 
     $n = 0;
     @lens = (76, 84, 94);
-    while (<Font::TTF::OS_2::DATA>)
+    while ($t = <Font::TTF::OS_2::DATA>)
     {
-        s/\r?\n$//;
-        if (m/^\s*$/)
+        $t =~ s/\r?\n$//;
+        if ($t =~ m/^\s*$/)
         {
             $n++;
             next;
         }
-        ($k, $v, $c) = TTF_Init_Fields($_, $c);
+        ($k, $v, $c) = TTF_Init_Fields($t, $c);
         next unless defined $k && $k ne "";
         for ($i = $n; $i < 3; $i++)
         { $fields[$i]{$k} = $v; }
@@ -142,19 +142,68 @@ sub out
 }
 
 
+=head2 $t->XML_element($context, $depth, $key, $value)
+
+Tidies up the hex values to output them in hex
+
+=cut
+
+sub XML_element
+{
+    my ($self) = shift;
+    my ($context, $depth, $key, $value) = @_;
+    my ($fh) = $context->{'fh'};
+
+    if ($key =~ m/^ul(?:Unicode|CodePage)Range\d$/o)
+    { $fh->printf("%s<%s>%08X</%s>\n", $depth, $key, $value, $key); }
+    elsif ($key eq 'achVendID')
+    { $fh->printf("%s<%s name='%s'/>\n", $depth, $key, pack('N', $value)); }
+    else
+    { return $self->SUPER::XML_element(@_); }
+    $self;
+}
+
+
+=head2 $t->XML_end($context, $tag, %attrs)
+
+Now handle them on the way back in
+
+=cut
+
+sub XML_end
+{
+    my ($self) = shift;
+    my ($context, $tag, %attrs) = @_;
+
+    if ($tag =~ m/^ul(?:Unicode|CodePage)Range\d$/o)
+    { return hex($context->{'text'}); }
+    elsif ($tag eq 'achVendID')
+    { return unpack('N', $attrs{'name'}); }
+    else
+    { return $self->SUPER::XML_end(@_); }
+}
+
 =head2 $t->update
 
+Updates the OS/2 table by getting information from other sources:
+
 Updates the C<firstChar> and C<lastChar> values based on the MS table in the
 cmap.
 
+Updates the sTypoAscender, sTypoDescender & sTypoLineGap to be the same values
+as Ascender, Descender and Linegap from the hhea table (assuming it is dirty)
+and also sets usWinAscent to be the sum of Ascender+Linegap and usWinDescent to
+be the negative of Descender.
+
 =cut
 
 sub update
 {
     my ($self) = @_;
-    my ($map, @keys);
+    my ($map, @keys, $table);
+
+    return undef unless ($self->SUPER::update);
 
-    return undef unless $self->{' PARENT'}{'cmap'}{' isDirty'};
     $self->{' PARENT'}{'cmap'}->update;
     $map = $self->{' PARENT'}{'cmap'}->find_ms || return undef;
 
@@ -162,6 +211,13 @@ sub update
 
     $self->{'usFirstCharIndex'} = $keys[0];
     $self->{'usLastCharIndex'} = $keys[-1];
+
+    $table = $self->{' PARENT'}{'hhea'};
+    $self->{'sTypoAscender'} = $table->{'Ascender'};
+    $self->{'sTypoDescender'} = $table->{'Descender'};
+    $self->{'sTypoLineGap'} = $table->{'Linegap'};
+    $self->{'usWinAscent'} = $self->{'sTypoAscender'} + $self->{'sTypoLineGap'};
+    $self->{'usWinDescent'} = -$self->{'sTypoDescender'};
     $self;
 }
 
@@ -214,7 +270,7 @@ fsSelection, S
 usFirstCharIndex, S
 usLastCharIndex, S
 sTypoAscender, S
-sTypoDescender, S
+sTypoDescender, s
 sTypoLineGap, S
 usWinAscent, S
 usWinDescent, S
